/*
 *  Copyright (C) 2003 Marco Pesenti Gritti
 *  Copyright (C) 2003 Christian Persch
 *  Copyright (C) 2004 Crispin Flowerday
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id$
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "ephy-view-source-extension.h"
#include "mozilla-helpers.h"

#include <epiphany/ephy-extension.h>
#include <epiphany/ephy-shell.h>
#include <gtk/gtkuimanager.h>
#include <gtk/gtkactiongroup.h>
#include <gtk/gtkstock.h>

#include <gmodule.h>
#include <string.h>

/* Use Epiphanys translations for the strings in here, no point in
 * doing it all twice! */
#define N_(a) a
#define EPHY_GETTEXT_PACKAGE "epiphany"
#define STOCK_VIEW_SOURCE          "stock_view-html-source"

static void ephy_view_source_extension_class_init	(EphyViewSourceExtensionClass *klass);
static void ephy_view_source_extension_iface_init	(EphyExtensionIface *iface);
static void ephy_view_source_extension_init		(EphyViewSourceExtension *extension);

static GObjectClass *parent_class = NULL;

static GType type = 0;

GType
ephy_view_source_extension_get_type (void)
{
	return type;
}

GType
ephy_view_source_extension_register_type (GTypeModule *module)
{
	static const GTypeInfo our_info =
	{
		sizeof (EphyViewSourceExtensionClass),
		NULL, /* base_init */
		NULL, /* base_finalize */
		(GClassInitFunc) ephy_view_source_extension_class_init,
		NULL,
		NULL, /* class_data */
		sizeof (EphyViewSourceExtension),
		0, /* n_preallocs */
		(GInstanceInitFunc) ephy_view_source_extension_init
	};

	static const GInterfaceInfo extension_info =
	{
		(GInterfaceInitFunc) ephy_view_source_extension_iface_init,
		NULL,
		NULL
	};

	type = g_type_module_register_type (module,
					    G_TYPE_OBJECT,
					    "EphyViewSourceExtension",
					    &our_info, 0);

	g_type_module_add_interface (module,
				     type,
				     EPHY_TYPE_EXTENSION,
				     &extension_info);

	return type;
}

static void
ephy_view_source_extension_init (EphyViewSourceExtension *extension)
{
}

static void
ephy_view_source_extension_finalize (GObject *object)
{

	G_OBJECT_CLASS (parent_class)->finalize (object);
}

#if EPHY_VERSION == 14
static GObject*
ephy_window_get_ui_manager (EphyWindow *window)
{
   return window->ui_merge;
}
#endif


static void
window_cmd_view_page_source (GtkAction *action,
			     EphyWindow *window)
{
	EphyEmbed *old_embed, *new_embed;
	EphyTab *old_tab, *new_tab;

	old_tab   = ephy_window_get_active_tab (window);
	old_embed = ephy_tab_get_embed (old_tab);
	
	new_tab = ephy_shell_new_tab (ephy_shell, window, old_tab,
				      "about:blank", 
				      EPHY_NEW_TAB_APPEND_AFTER|
				      EPHY_NEW_TAB_JUMP|
				      EPHY_NEW_TAB_IN_EXISTING_WINDOW|
				      EPHY_NEW_TAB_OPEN_PAGE);

	new_embed = ephy_tab_get_embed (new_tab);
	mozilla_view_source (old_embed, new_embed);

#if EPHY_VERSION >= 18
        gtk_widget_grab_focus (GTK_WIDGET (new_embed));
#endif
}


static void
window_cmd_reload (GtkAction *action,
                   EphyWindow *window)
{
	EphyEmbed *embed;
        char * location;

	embed = ephy_window_get_active_embed (window);
        g_return_if_fail (EPHY_IS_EMBED (embed));

        location = ephy_embed_get_location (embed, TRUE);
        if (location && g_str_has_prefix (location, "view-source:"))
	{
		/* Use our own reload routine, so we can force 
		 * the reload, in theory, we could replace the
		 * reload() function pointer in the EphyEmbed
		 * interface with our own, and just set the 'force'
		 * flag, but this is less of a hack IMO */
		mozilla_reload_source (embed);

		g_signal_stop_emission_by_name (action, "activate");
	}
        g_free (location);
}

static const GtkActionEntry actions[] = {

	{ "ViewReload", GTK_STOCK_REFRESH, N_("_Reload"), "<control>R",
	  N_("Display the latest content of the current page"),
	  G_CALLBACK (window_cmd_reload) },
	{ "ViewPageSource", STOCK_VIEW_SOURCE, N_("_Page Source"), "<control>U",
	  N_("View the source code of the page"),
	  G_CALLBACK (window_cmd_view_page_source) },
};


static void
impl_attach_window (EphyExtension *ext,
		    EphyWindow *window)
{
	GtkUIManager *manager;
	GtkActionGroup *group;

	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager(window));
	g_return_if_fail (GTK_IS_UI_MANAGER (manager));

        group = gtk_action_group_new("ViewSourceExtensionGroup");
	gtk_action_group_set_translation_domain (group, EPHY_GETTEXT_PACKAGE);
        gtk_action_group_add_actions (group, actions, 
                                      G_N_ELEMENTS (actions), window);

        gtk_ui_manager_insert_action_group (manager, group, 0);
	g_object_unref (group);
}

static void
impl_detach_window (EphyExtension *ext,
		    EphyWindow *window)
{
	GtkUIManager *manager;
        GList *list, *li;

	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager(window));
	g_return_if_fail (GTK_IS_UI_MANAGER (manager));

        list = gtk_ui_manager_get_action_groups (manager);
        for (li = list ; li ; li = li->next)
	{
		GtkActionGroup *group = GTK_ACTION_GROUP (li->data);

		if (!strcmp (gtk_action_group_get_name (group), "ViewSourceExtensionGroup"))
		{
			gtk_ui_manager_remove_action_group (manager, group);
			break;
		}
	}
}

static void
ephy_view_source_extension_iface_init (EphyExtensionIface *iface)
{
	iface->attach_window = impl_attach_window;
	iface->detach_window = impl_detach_window;
}

static void
ephy_view_source_extension_class_init (EphyViewSourceExtensionClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);

	object_class->finalize = ephy_view_source_extension_finalize;
}
